/* FlyStats is copyright (c) 1999-2000 CRESTO Sylvain <tost>.
 * E-mail: <tost@softhome.net>
 * Official channel: irc.kewl.org on #help
 *
 * This program is free but copyrighted software; see the file COPYING for
 * details.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define INDEXFILE	"index"
#define HEADERFILE	"../src/language.h"

typedef struct language_ Language;
struct language_ {
    Language *next, *prev;
    char *topic;
    int nbrline;
    char **content;
};

Language *langbuf = NULL, *LastBuf = NULL;
int num_topic, warning;

int write_int16(int val, FILE *f)
{
    if (fputc((val>>8) & 0xFF, f) == EOF || fputc(val & 0xFF, f) == EOF)
	return 0;
    return 1;
}

int write_string(const char *s, FILE *f)
{
    int len;

    if (!s)
	return (write_int16(0, f) != EOF);
    len = strlen(s);
    if (len > 512)
	len = 512;
    if (write_int16((int)(len+1), f) < 0)
	return 0;
    if (len && fwrite(s, 1, len, f) != len)
	return 0;
    return (fputc(0, f) != EOF);
}

int loadtopic()
{
    FILE *f = fopen(INDEXFILE, "r");
    char buf[512];
    int i, l=0, nt_tmp=0;
    Language *tmplang;

    num_topic = 0;
    if (!f) {
	printf("makelang: Cannot open '%s'\n", INDEXFILE);
	return 0;
    }
    while (fgets(buf, 512, f)) {
	l++;
	i = strlen(buf)-1;
	while (buf[i] && buf[i]<32)
	    buf[i--]=0;
	if (!*buf) {
	    printf("makelang: Warning (%s-%d): line is NULL\n", INDEXFILE, l);
	    warning = 1;
	    continue;
	}
	for (tmplang = langbuf, i=1; tmplang; tmplang = tmplang->next, i++)
	if (!strcmp(tmplang->topic, buf)) {
   	    printf("makelang: Warning (%s-%d): topic '%s' already defined on line %d\n", INDEXFILE, buf, i);
	    warning = 1;
	    i=0;
	    break;
	}
	if (!i)
	    continue;
	nt_tmp++;
	tmplang = malloc(sizeof(*tmplang));
	tmplang->topic = strdup(buf);
	tmplang->nbrline = 0;
	tmplang->content = NULL;
	tmplang->next = NULL;
	tmplang->prev = LastBuf;
	if (LastBuf)
	    LastBuf->next = tmplang;
	LastBuf = tmplang;
	if (!langbuf)
	    langbuf = LastBuf;
    }
    fclose(f);
    if (!num_topic)
	num_topic=nt_tmp;
    return 1;
}

int openlang(char *lang)
{
    FILE *f = fopen(lang, "r");
    char buf[512], *s;
    int i, l=0, dt=0;
    Language *tmplang = NULL;

    if (!f) {
	printf("makelang: Cannot open '%s'\n", lang);
	return 0;
    }
    while (fgets(buf, 512, f)) {
	l++;
	i = strlen(buf)-1;
	while (*buf && buf[i]<32)
	    buf[i--]=0;
	if (!*buf || *buf=='#')
	    continue;
	if (*buf!=9) {
	    /* topic */
	    for (tmplang = langbuf, i=1; tmplang; tmplang = tmplang->next, i++)
	    if (!strcmp(tmplang->topic, buf))
		break;
	    if (!tmplang) {
		printf("makelang: Warning (%s-%d): Unknow title '%s'\n", lang, l, buf);
		warning = 1;
	    } else if (tmplang->nbrline) {
		printf("makelang: Warning (%s-%d): Title '%s' already defined, ignoring last\n", lang, l, buf);
		warning = 1;
		tmplang = NULL;
	    } else {
		dt=1;
	    }
        } else {
	    if (tmplang) {
		char *d;
		if (tmplang->nbrline<100) {
		    tmplang->content = realloc(tmplang->content, sizeof(char *) * ++tmplang->nbrline);
		    s = buf;
		    while (*s==9)
			s++;
		    d = s;
		    while (*d) {
			if (*d=='\\') {
			    *d=2;
			} else if (*d=='@') {
			    *d=31;
			}
			d++;
		    }
		    tmplang->content[tmplang->nbrline-1] = strdup(s);
		} else {
		    printf("makelang: Warning (%s-%d): They are too many line (99 max) on topic `%s`", lang, l, tmplang->topic);
		    warning = 1;
		}
	    } else if (!dt) {
		printf("makelang: Warning (%s-%d): Found text before topic, ignoring it\n", lang, l, buf);
		warning = 1;
	    }
	}
    }
    fclose(f);
    /* verifis pour des topic manquants */
    for (tmplang = langbuf; tmplang; tmplang = tmplang->next)
    if (!tmplang->nbrline) {
	printf("makelang: Warning (%s): Topic %s is missing\n", lang, tmplang->topic);
	warning = 1;
    }
    return 1;
}

int storelang(char *lang)
{
    FILE *f = fopen(lang, "w");
    Language *tmplang;
    int i;
    char buf[10000];
    if (!f) {
	printf("makelang: Cannot save language '%s'\n", lang);
	return 0;
    } else {
	write_int16(num_topic, f);
	for (tmplang = langbuf, i=0; tmplang; tmplang = tmplang->next, i++) {
	    *buf = 0;
	    for (i=0; i<tmplang->nbrline; i++) {
		strcat(buf, tmplang->content[i]);
		if (i< tmplang->nbrline-1)
		    strcat(buf, "\n");
	    }
	    strcat(buf, "\r");
	    write_int16(strlen(buf), f);
	    fwrite(buf, 1, strlen(buf), f);
	}
	fclose(f);
    }
    return 1;
}

void freelang()
{
    Language *tmplang, *nextlang;
    int i;

    for (tmplang = langbuf; tmplang; tmplang = nextlang) {
	free(tmplang->topic);
	tmplang->topic = NULL;
	for (i=0; i<tmplang->nbrline; i++)
	    free(tmplang->content[i]);
	if (tmplang) {
	    nextlang = tmplang->next;
	    free(tmplang);
	    tmplang = NULL;
	} else nextlang = NULL;
    }
    langbuf = NULL;
}

int createheader()
{
    FILE *f = fopen(HEADERFILE, "w");
    Language *tmplang;
    int i;
    char buf[512];

    if (!f) {
        printf("makelang: ERROR: Cannot create '%s' file.\n", HEADERFILE);
        return 0;
    }
    for (tmplang = langbuf, i=0; tmplang; tmplang = tmplang->next, i++) {
	sprintf(buf, "#define %s\t\t\t%d\n", tmplang->topic, i);
	fputs(buf, f);
    }
    fclose(f);
    printf("makelang: Header file '%s' successfuly created.\n", HEADERFILE);
    return 1;
}

/* si ch == 1 alors on cr le fichier header language.h */
void processlang(char *lang, int ch)
{
    warning = 0;
    if (!loadtopic()) {
	printf("makelang: Skipping language %s\n", lang);
    } else if (openlang(lang)) {
	char buf[512], *s;
	strcpy(buf, lang);
	if (s = strrchr(buf, '.')) {
	    *(s++)=0;
	    storelang(s);
	} else {
	    printf("makelang: '%s' invalid file name.\n", lang);
	}
	if (!warning) {
	    printf("makelang: '%s' language file created.\n", s);
	} else {
	    printf("makelang: '%s' language file created with warning! file might be damaged.\n", s);
	}
	if (ch)
	    createheader();
	freelang();
    }
}

int main()
{
    processlang("txt.flyfr", 1);
    processlang("txt.flyuk", 0);
//  processlang("txt.flynl", 0);
    return 0;
}
