/* FlyStats! is copyright (c) 1999-2000 CRESTO Sylvain <tost>.
 * E-mail: <tost@softhome.net>
 * Contact: #flystats on irc.kewl.org
 *
 * This program is free but copyrighted software; see the file COPYING for
 * details.
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <pthread.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <string.h>
#include <stdarg.h>
#include <ctype.h>
#include <sys/time.h>
#include "config.h"
#include "declare.h"
#include "language.h"
#include "sysconf.h"

#define flystats "0.9-7"
#define flyname	"La rage"

#if defined(IRC_UNDERNET) || defined(IRC_UNDERNET_29) || defined (IRC_HYBRID)
#define WALLOPS "WALLOPS"
#else
#define WALLOPS "GLOBOPS"
#endif

/* commands.c */
extern void run_cmd(char *service, User *u, Commands *list, const char *cmd);
extern void run_help(char *service, User *u, Commands *list, int helptopic);

/* config.c */
extern int OPERONLY;
extern int ACCESS;
extern RemoteServer *FirstRemoteServer;
extern RemoteServer *UsedRemoteServer;
extern AccessFlags *accessflags;
extern char *ServerName;
#ifdef IRC_UNDERNET
extern int  ServerNumeric;
#endif
extern int SleepBeforeReconnect;
#ifdef IRC_UNREAL
extern char *IrcNetwork;
#endif
extern int  RotateServer;
extern char *ServerDesc;
extern char *AdminInfo;

extern int  ServerLagTime;
extern int  WallOnOperCommand;
extern int  WallOnIdentify;
extern int  WallOnFailedIdentify;
#ifdef USETHREAD
extern int  ScanForProxy;
extern int  MaxThreadForProxyScan;
extern int  ScanForWhat;
#endif

extern char *ssNick;
extern char *ssUser;
extern char *ssHost;
extern char *ssRealName;

#ifdef IRC_UNREAL
extern char *bakssNick;
extern char *bakssUser;
extern char *bakssHost;
extern char *bakssRealName;
extern char *bakServerDesc;
#endif

#ifdef USETHREAD
extern char *ProxyNotice;
extern char *KillReason;
extern char *AKillCommand;
#endif

extern char **IgnoreServerList;
extern int  IgnoreServerCount;

extern int  PingServerTime;
extern int  UpdateStatTime;
extern int  UpdateChanStatTime;
extern int  OperOnly;
extern int  NotifyOpersMissingServer;
extern int  WallOnNewOper;
extern int  LogMaxUser;

extern int  DetectLangCount;
extern int  DefaultLang;

extern int  fRemoveServer;
extern int  fRehash;
extern int  fRestart;
extern int  fShutDown;

extern int load_config();

/* country.c */
extern char *find_country(char *host);

/* domain.c */
extern char *DataDirectory;
extern int domaincount;
extern int resolvecount;
extern Domain *new_domain(char *name);
extern Domain *find_domain(const char *name);
extern void domain_adduser(Domain *domain);
extern void update_domain_stats(int timepos, int rzero);
Domain *firstdomain(void);
Domain *nextdomain(void);

/* file.c */
extern int read_int16(int *ret, FILE *f);
extern int write_int16(int val, FILE *f);
extern int read_string(char **ret, FILE *f);
extern int write_string(const char *s, FILE *f);

/* flymisc.c */
extern char flyflags[10];
extern void do_flyflags(void);
extern void update_flyflags(void);

/* help.c */
extern void show_help(char *prog);
extern void do_copyright();
extern void do_info();

#ifdef IRC_UNDERNET
/* ircu.c */
extern const char convert2y[];
extern char *inttobase64(char *buf, unsigned int v, size_t count);
#endif

/* language.c */
extern char *filelang[];
extern char **FlyLang[NUM_LANGS];
extern int LanguageNum;
extern AutoDetectLang *detectlanglist[255];
extern void new_detectlang(char *host, int lang);
extern int detectlang(const char *host);
extern void free_detectlang();
extern void notice_lang(char *source, User *u, int message, ...);
extern void syntax_error(char *source, User *u, char *cmd, int message);
#define getstring(u, index) \
	(FlyLang[u ? u->language : DefaultLang][(index)])
extern void load_lang();
extern int findlang_by_file(char *name);
#ifdef IRC_UNDERNET
#define nickyxx(u) u->yxx
#else
#define	nickyxx(u) u->nick
#endif

/* log.c */
extern int log(char *fmt,...);
extern void close_log();

/* main.c */
extern time_t runsince;
extern struct tm runsince_tm;
#ifdef ALLOW_DEBUG
extern int debug;
#endif
extern int nofork;
extern int statrun;
extern int serversock;
extern int quitflystats;
extern int num_command;
extern int tolerance;
extern char *RemoteServerIp;
extern void connect_client(char *client);
#ifdef IRC_UNREAL
extern void update_client(char *client);
#endif
extern int pseudo_client(char *nick);
extern void quit_client(char *client, char *reason);

/* misc.c */
extern void trim(char *buf);
extern void trimleft(char *buf);
extern void trimright(char *buf);
extern void trim(char *buf);
extern int ispaire(char *buf, int car);
extern void trimex(char *buf, int car);
extern time_t dotime(char *buf, int unit);
extern int split_buf(char *bufori, char **source, char ***argv);
extern int match_wild(const char *pattern, const char *str);
extern int match_wild_nocase(const char *pattern, const char *str);
extern int truematch_wild(const char *pattern, const char *str);
extern char *strnrepl(char *s, int size, const char *old, const char *new);

/* proxy.c */
#ifdef USETHREAD
extern User *firstproxycheckuser, *lastproxycheckuser;
extern int proxythread_count;
extern UsingProxyThread *init_proxythread_struct(User *u);
extern UsingProxyThread *remove_proxythread_struct(UsingProxyThread *upt);
extern void do_proxycheck();

extern ProxyHost *new_proxyhost(char *host, int action);
extern ProxyHost *find_proxy(const char *host);
extern void add_proxycheck(User *u);
extern void do_OneProxyAction(char *nick, char *user, char *host, int action);
extern void do_ProxyAction();
#endif

/* serv_message.c */
extern char SERV[256];
extern Command *CommandsBuf;
extern Command *LastBuf;
extern Message *find_message(char *name);

/* server.c */
extern Server *flystat;
extern char *RemoteName;
#ifdef IRC_UNDERNET
extern char yxx[4];
Server *find_server_yxx(Server *server, char name);
#endif
extern int servernum;
extern int servercount;
extern int dailysplitcount;
extern int dailyservkillcount;
extern int maxservercount;
extern int showallservercount;
extern int actualsplit;
extern long splitcount;
extern long totalservkillcount;
extern time_t timemaxservercount;
extern long netaveragetotal[4];
extern long netaveragecount[4];
extern long netaverage[4];
extern Server *servmap;
extern void show_server(char *source, User *u, Server *server, char *view);
extern int show_all_servers(char *source, User *u, Server *server, char *mask);
extern Server *find_server(Server *server, char *name);
extern Server *find_server_match(Server *server, char *name);
extern int do_server(char *source, int c, char **cmd);
extern int do_squit(char *name);
extern void ping_server();
extern void do_pong(char *name);
extern int server_lag(Server *server);
extern void update_server_stats(Server *server, int timepos, int rzero);
extern void update_net_stats(int timepos, int rzero);
extern Server *remove_server_leaf(Server *server);
extern Server *squit_server_leaf(Server *server);
extern int IgnoreServer(Server *server);
extern void ctcp_reply(char *source, char *dest, char *cmd);
#ifdef IRC_UNDERNET
extern void remove_server_users(Server *server);
#endif
extern void remove_one_server(Server *server);

/* signal.c */
extern void setsignal();
extern void sighandler(int signum);

/* sock.c */
#ifdef IRC_UNDERNET
char ip_buf[8];
#endif
extern int ConnectFlyStats();
extern int ConnectTo(char *host, int port, struct sockaddr_in *sares);
extern void SockWrite(char *fmt,...);
extern int SockRead(char *c ,int s);

/* statserv.c */
extern long ss_helpcount;
extern long ss_mapcount;
extern long ss_serverscount;
extern long ss_servercount;
extern long ss_domaincount;
extern long ss_netstatscount;
extern long ss_operlistcount;
extern long ss_languagecount;
extern long ss_identifycount;
extern long ss_removeservercount;
extern long ss_rehashcount;
extern void do_statserv(char *source, int c, char *buf);
extern void do_ss_helpops(User *u, char *topic);

/* users.c */
extern char **operlist;
extern long usercount;
extern long channelcount;
extern long invusercount;
extern long maxusercount;
extern long maxchannelcount;
extern long totalkillcount;
extern long dailymaxusercount;
extern long dailytotalusercount;
extern long dailymaxchannelcount;
extern int opercount;
extern int maxopercount;
extern int dailykillcount;
extern time_t maxusertime;
extern time_t maxchanneltime;
extern time_t maxopertime;
extern time_t dailymaxusertime;
extern time_t dailymaxchanneltime;
extern User *identified;

extern User *finduser(const char *nick);
extern User *finduser_yxx(const char *nick);
extern void remove_ircop(User *u);
extern User *do_newnick(char **data);
extern void do_changenick(char *source, char *nick);
extern User *finduser(const char *nick);
extern void do_quit(char *nick, char *quitmessage);
extern void do_umode(char *nick, char *mode);
extern int is_oper(User *u);
extern int is_access(User *u, int flagrequired);
extern void User_UnIdentify(User *u);
extern int User_Identify(User *u, char *login, char *pass);
#ifndef IRC_UNDERNET
extern void remove_server_users(Server *server);
#endif
