/* FlyStats! is copyright (c) 1999-2000 CRESTO Sylvain <tost>.
 * E-mail: <tost@softhome.net>
 * Contact: #flystats on irc.kewl.org
 *
 * This program is free but copyrighted software; see the file COPYING for
 * details.
 */

#include "extern.h"

/* Enleve tous les carateres plus petit que 33 (espace, tab, cr..) a gauche (04/11/99)
 */

void trimleft(char *buf)
{
    char *s = buf;

    while (*s && *s<33)
        s++;
    if (s!=buf)
	memcpy(buf, s, strlen(s)+1); 
}

/* Enleve tous les carateres plus petit que 33 (espace, tab, cr..)  droite (04/11/99)
 */ 
void trimright(char *buf)
{
    char *s = strchr(buf, 0);
    
    if (!*buf || !s)
	return;
    s--;

    while (*s && *s<33)
	s--;
    if (*++s<33)
	*s=0;
}

/* Enleve tous les carateres plus petit que 33 (espace, tab, cr..) (05/11/99)
 */
void trim(char *buf)
{
    trimleft(buf);
    trimright(buf);
}

/* Renvois 1 si le buf contient un nombre paire d'un certain caractere, sinon 0) (05/11/99)
 */
int ispaire(char *buf, int car)
{
    char *s = buf;
    int is = 1;

    while (*s) {
	if (*s==car)
	    is =! is;
	s++;
    }
    return is;
}

/* Enleve un certain caractere d'une chaine (05/11/99)
 */
void trimex(char *buf, int car)
{
    char *s = buf;
    char *d = buf;
    int fc = 0;

    while (*s) {
	if (isalnum && !fc)
	    fc = 1;

        if (*s!=car) {
            *d++=*s;
	    fc = 0;
	}
        s++;
    }
    if (s!=d)
        *d=0;
}

/* Convertis une chaine en temps
 * s = seconde, m = minute, h = heure, d = jours
 * si unit == 0 l'unit utilis doit etre indiqu dans la chaine sinon unit est l'unit par defaut
 * renvois -1 si y'a une erreeur
 */

time_t dotime(char *buf, int unit)
{
    char *s = buf + strlen(buf) - 1;
    int cu = 0;
    time_t stime;

    if (*s=='s' || *s=='m' || *s=='h' || *s=='d') {
	cu = *s;
	*s=0;
    }
    if ((!cu && !unit) || (strspn(buf, "0123456789") != strlen(buf))) {
	return -1;
    }
    if (!cu) {
	cu = unit;
    }

    stime = atol(buf);
    switch (cu) {
	case 's': return stime;		/* secondes */
	case 'm': return stime * 60;	/* minutes */
	case 'h': return stime * 3600;	/* heures */
    }
    return stime * 86400;		/* jours */
}

int split_buf(char *bufori, char **source, char ***argv)
{
    int argc = 0, argsize = 8, sender = 0;
    char *s=bufori, *b=bufori;

    *argv = malloc(sizeof(char *) * argsize);
    (*source) = NULL;

    while (*b) {
	if (argc == argsize) {
	    argsize += 8;
	    *argv = realloc(*argv, sizeof(char *) * argsize);
	}
	if (*b==' ') {
	    *b=0;
	    if (!sender) {
#ifdef IRC_UNDERNET
		if (s==bufori && (s+1==b || s+3==b)) {
		    (*source) = s;
		} else
#endif
		{
		    (*argv)[argc++] = s;
		}
	    } else {
		sender = 0;
		(*source) = s;
	    }
	    s = b+1;
	} else if (*b==':') {
	    if (b++==bufori) {
		sender = 1;
		s++;
	    } else {
		(*argv)[argc++] = b;
		return argc;
	    }
	}
	b++;
    }
    if (*s) {
	if (!sender) {
	    if (argc == argsize) {
		*argv = realloc(*argv, sizeof(char *) * ++argsize);
	    }
	    (*argv)[argc++] = s;
	} else {
	    (*source) = s;
	}
    }
    return argc;
}

static int do_match_wild(const char *pattern, const char *str, int docase)
{
    char c;
    const char *s;

    for (;;) {
	switch (c = *pattern++) {
	  case 0:
	    if (!*str)
		return 1;
	    return 0;
	  case '?':
	    if (!*str)
		return 0;
	    str++;
	    break;
	  case '*':
	    if (!*pattern)
		return 1;
	    s = str;
	    while (*s) {
		if ((docase ? (*s==*pattern) : (tolower(*s)==tolower(*pattern)))
				&& do_match_wild(pattern, s, docase))
		    return 1;
		s++;
	    }
	    break;
	  default:
	    if (docase ? (*str++ != c) : (tolower(*str++) != tolower(c)))
		return 0;
	    if (docase == 2 && (*str=='*' || *str=='?')) str++;
	    break;
	}
    }
}

int match_wild(const char *pattern, const char *str)
{
    return do_match_wild(pattern, str, 1);
}

int match_wild_nocase(const char *pattern, const char *str)
{
    return do_match_wild(pattern, str, 0);
}

int truematch_wild(const char *pattern, const char *str)
{
    return do_match_wild(pattern, str, 2);
}

/* Remplace une chaine de caractere par une autre */
char *strnrepl(char *s, int size, const char *old, const char *new)
{
    char *ptr = s;
    int left = strlen(s);
    int avail = size - (left+1);
    int oldlen = strlen(old);
    int newlen = strlen(new);
    int diff = newlen - oldlen;

    while (left >= oldlen) {
	if (!strncmp(ptr, old, oldlen)) {
	    left--;
	    ptr++;
	    continue;
	}
	if (diff > avail)
	    break;
	if (diff)
	    memmove(ptr+oldlen+diff, ptr+oldlen, left+1);
	strncpy(ptr, new, newlen);
	ptr += newlen;
	left -= oldlen;
    }
    return s;
}
