/* FlyStats! is copyright (c) 1999-2000 CRESTO Sylvain <tost>.
 * E-mail: <tost@softhome.net>
 * Contact: #flystats on irc.kewl.org
 *
 * This program is free but copyrighted software; see the file COPYING for
 * details.
 */

#include "extern.h"
#ifdef USETHREAD

#define HASH(host)	(((host)[0]&31)<<5 | ((host)[1]&31))
ProxyHost *proxyhostlist[1024];

/* Nouvelle gestion de la detection des utilisateurs */
UsingProxyThread *firstproxythread, *lastproxythread = NULL;
User *firstproxycheckuser, *lastproxycheckuser = NULL;
int proxythread_count = 0;
static void checking_for_proxy(UsingProxyThread *upt);

UsingProxyThread *init_proxythread_struct(User *u)
{
    UsingProxyThread *upt = malloc(sizeof(UsingProxyThread));

    upt->nick = strdup(u->nick);
    upt->username = strdup(u->username);
    upt->host = strdup(u->host);
    upt->action = 0;	/* 0 = test en cours */
    upt->next = NULL;
    upt->prev = lastproxythread;
    if (lastproxythread) {
	lastproxythread->next = upt;
    } else firstproxythread = upt;
    lastproxythread = upt;
    proxythread_count++;
    return upt;
}

UsingProxyThread *remove_proxythread_struct(UsingProxyThread *upt)
{
    UsingProxyThread *retupt = upt->next;
#ifdef ALLOW_DEBUG
    if (debug)
	log("Removing '%s' from proxythread", upt->nick);
#endif
    if (upt->next) {
	upt->next->prev = upt->prev;
    } else lastproxythread = upt->prev;
    if (upt->prev) {
	upt->prev->next = upt->next;
    } else firstproxythread = upt->next;
    free(upt->nick);
    free(upt->username);
    free(upt->host);
    free(upt);
    proxythread_count--;
    return retupt;
}

void do_proxycheck()
{
    pthread_t thread;
    UsingProxyThread *upt;
    User *u;

	/* on traite les thread dj existantes */
    for (upt = firstproxythread; upt; upt = upt->next) {
	/* upt->action ->
	 * 0 = test en cours
	 * 1 = pas de proxy
	 * 2 = socks5
	 * 4 = socks4
	 * 8 = wingate
	 * 16 = + cached
	 */
#ifdef ALLOW_DEBUG
	if (debug)
	    log("In proxycheck() %s!%s@%s -> %i", upt->nick, upt->username, upt->host, upt->action);
#endif
	if (upt->action) {
	    if (upt->action > 1)
		do_OneProxyAction(upt->nick, upt->username, upt->host, upt->action);
	    if (!(upt = remove_proxythread_struct(upt)))
		break;
	}
    }
    /* on prend des users a tester et on lance les nouveaux threads */
    while ((u = firstproxycheckuser) && proxythread_count < MaxThreadForProxyScan) {
	upt = init_proxythread_struct(u);
	firstproxycheckuser = u->proxy_next;
	u->proxy_next = u->proxy_prev = NULL;
	pthread_create(&thread, NULL, (void *)checking_for_proxy, (void *)upt);
    }
    if (!firstproxycheckuser)
	lastproxycheckuser = NULL;
}

/* gestion du cache de detection des proxy */
ProxyHost *new_proxyhost(char *host, int action)
{
    ProxyHost *proxyhost = malloc(sizeof(ProxyHost)), **list;

    log("Added '%s' in ProxyHost cache", host);

    list = &proxyhostlist[HASH(host)];
    proxyhost->next = *list;
    if (*list)
	(*list)->prev = proxyhost;
    *list = proxyhost;
    proxyhost->host = strdup(host);
    proxyhost->prev = NULL;
    proxyhost->action = action;
    return proxyhost;
}

ProxyHost *find_proxy(const char *host)
{
    ProxyHost *proxyhost = proxyhostlist[HASH(host)];

    while (proxyhost && strcasecmp(proxyhost->host, host))
        proxyhost = proxyhost->next;
    return proxyhost;
}

/* Thread qui scan pour un proxy/wingate */
void checking_for_proxy(UsingProxyThread *upt)
{
    struct hostent *hp;
    struct sockaddr_in sa;
    int sock = -1;
    time_t statim = time(NULL);
    static unsigned char packet[12];

    if (!(hp = gethostbyname (upt->host)))
	return;

	/* test pour un proxy socks 5 sur le port 1080 */
    if (ScanForWhat & 1) {
	if ((sock = socket(AF_INET, SOCK_STREAM, 0)) == -1)
	    return;
	bzero(&sa, sizeof(sa));
	sa.sin_family = AF_INET;
	sa.sin_port = htons (SOCKS5PORT);
	bcopy(hp->h_addr, (char *) &sa.sin_addr, hp->h_length);
	if (connect(sock, (struct sockaddr *) &sa, sizeof (sa))!=-1) {
	    packet[0] = 5; packet[1] = 1; packet[2] = 0;
	    if (write(sock, packet, 3) == 3) {
		bzero(packet, 2);
		if (read(sock, packet, 2) == 2 && packet[0]==5 && packet[1]==0) {
		    close(sock);
		    upt->action = 2;	/* 2 = socks5 */
		    return;
		}
	    }
	    close(sock);
		/* test pour un proxy sovks 4 sur le port 1080 */
	    if ((sock = socket(AF_INET, SOCK_STREAM, 0)) == -1)
		return;
	    bzero(&sa, sizeof(sa));
	    sa.sin_family = AF_INET;
	    sa.sin_port = htons (SOCKS4PORT);
	    bcopy(hp->h_addr, (char *) &sa.sin_addr, hp->h_length);
	    if (connect(sock, (struct sockaddr *) &sa, sizeof (sa))!=-1) {
		unsigned long ip = htonl(sa.sin_addr.s_addr);
		packet[0] = 4; packet[1] = 1; packet[2] = (((unsigned short) SOCKS4PORT) >> 8);
		packet[3] = (((unsigned short) SOCKS4PORT) & 255); packet[4] = (ip >> 24);
		packet[5] = ((ip >> 16) & 255); packet[6] = ((ip >> 8) & 255);
		packet[7] = (ip & 255); packet[8] = 0;
		if (write(sock, packet, 9) == 9) {
		    bzero(packet, 9);
		    if (read(sock, packet, 9)>=4 && packet[1] == 90) {
			close(sock);
			upt->action = 4;	/* 4 = socks4 */
			return;
		    }
		}
		close(sock);
	    }
	}
    }
	/* test pour un wingate */
    if ((ScanForWhat & 2) && statim < time(NULL) + 60) {
	if ((sock = socket(AF_INET, SOCK_STREAM, 0)) == -1)
	    return;
	bzero(&sa, sizeof(sa));
	sa.sin_family = AF_INET;
	sa.sin_port = htons (WINGATE);
	bcopy(hp->h_addr, (char *) &sa.sin_addr, hp->h_length);
	if (connect(sock, (struct sockaddr *) &sa, sizeof (sa))!=-1) {
	    memset(packet, 0, sizeof(packet));
	    if (read(sock, packet, 8)==8 && (!strncasecmp(packet, "WinGate>", 8) || !strncasecmp(packet, "Too many", 8))) {
		close(sock);
		upt->action = 8;	/* 8 = wingate */
		return;
	    }
	    close(sock);
	}
    }
    upt->action =1;	/* 1 = pas de proxy */
}


/* valeur de proxyaction
 *   2 - socks5
 *   4 - socks4
 *   8 - wingate
 *   16 - cached
 */
void do_OneProxyAction(char *nick, char *user, char *host, int action)
{
    char buf[512];
    User *u;

#ifdef IRC_UNDERNET
    if (!(u=finduser(nick))) {
	log("Unknown user '%s!%s@%s' used an insecure wingate/proxy", nick, user, host);
	return;
    }
#endif
    SockWrite(":%s NOTICE %s :%s", ssNick, 
#ifdef IRC_UNDERNET
	u->yxx,
#else
	nick,
#endif
	ProxyNotice);
    if (ScanForProxy & 1) {
	if (action & 2) {
	    strcpy(buf, "socks5");
	} else if (action & 4) {
	    strcpy(buf, "socks4");
	} else if (action & 8) {
	    strcpy(buf, "wingate");
	}
	if (action & 16) {
	    strcat(buf, " - cached");
	}
        log("Found an insecure proxy '%s!%s@%s' (%s)", nick, user, host, buf);
	SockWrite(":%s %s :WARNING! %s!%s@%s is an \2Insecure proxy (%s) !\2", ssNick, WALLOPS,
		nick, user, host, buf);
    }
    if (ScanForProxy & 2) {
	strncpy(buf, AKillCommand, 512);
	strnrepl(buf, 512, "$N", nick);
	strnrepl(buf, 512, "$U", user);
	strnrepl(buf, 512, "$H", host);
	SockWrite(buf);
    }
    if (ScanForProxy & 4) {
#ifndef IRC_UNDERNET
	if ((u = finduser(nick))) {
#endif
	    if (u->mode & UMODE_O) {
		SockWrite(":%s %s :'%s' is an IRC Operator and he's using an insecure proxy (?)", ssNick, WALLOPS, nick);
		log("'%s' is an IRC Operator and he's using an insecure proxy (?)", nick);
		return;
	    }
	    do_quit(nick, KillReason);
	    if (u->server)
		u->server->killcount++;
	    totalkillcount++;
	    dailykillcount++;
#ifndef IRC_UNDERNET
	} else {
	    log("do_ProxyAction: Argh! We should KILL an unknown user '%s!%s@%s'", nick, user, host);
	}
#endif
	SockWrite(":%s KILL %s :%s", ssNick, nick, KillReason);
    }
}
#endif
