/* FlyStats! is copyright (c) 1999-2000 CRESTO Sylvain <tost>.
 * E-mail: <tost@softhome.net>
 * Contact: #flystats on irc.kewl.org
 *
 * This program is free but copyrighted software; see the file COPYING for
 * details.
 */

#include "extern.h"

long ss_helpcount = 0;
long ss_mapcount = 0;
long ss_serverscount = 0;
long ss_servercount = 0;
long ss_domaincount = 0;
long ss_netstatscount = 0;
long ss_operlistcount = 0;
long ss_languagecount = 0;
long ss_identifycount = 0;
long ss_removeservercount = 0;
long ss_connectcount = 0;
long ss_rehashcount = 0;

static void stat_time(char *source, User *u);
static void do_map(User *u);
static void do_servers(User *u);
static void do_ss_server(User *u);
static void do_domain(User *u);
static void do_netstats(User *u);
static void do_operlist(User *u);
static void do_set_language(User *u);
static void do_identify(User *u);
static void do_removeserver(User *u);
static void do_rehash(User *u);
static void do_restart(User *u);
static void do_shutdown(User *u);

static Commands cmds[] = {
	{ "HELP",	NULL,	NULL,	STAT_HELP,	-1,	-1 },
        { "MAP",	do_map,	NULL,	STAT_HELP_MAP,	-2,	-2 },
        { "SERVERS",    do_servers, NULL,	STAT_HELP_SERVERS,	-2,	-2 },
	{ "SERVER",	do_ss_server, NULL, STAT_HELP_SERVER,	STAT_SERVER_SYNTAX,	STAT_HELP_SERVER_EXAMPLE },
	{ "DOMAIN",	do_domain, NULL,	STAT_HELP_DOMAIN,	STAT_DOMAIN_SYNTAX,	STAT_HELP_DOMAIN_EXAMPLE },
	{ "NETSTATS",	do_netstats, NULL, STAT_HELP_NETSTATS,	-2,	-2 },
	{ "OPERLIST",   do_operlist, NULL, STAT_HELP_OPERLIST,	-2,	-2 },
	{ "LANGUAGE",	do_set_language, NULL, STAT_HELP_LANGUAGE,	STAT_LANGUAGE_SYNTAX,	STAT_HELP_LANGUAGE_EXAMPLE },
	{ "IDENTIFY",	do_identify,	&OPERONLY,	STAT_HELP_IDENTIFY,	STAT_IDENTIFY_SYNTAX,  STAT_HELP_IDENTIFY_EXAMPLE },
	{ "REMOVESERVER", do_removeserver, &fRemoveServer, STAT_HELP_REMOVESERVER, STAT_REMOVESERVER_SYNTAX,
												STAT_HELP_REMOVESERVER_EXAMPLE },
	{ "REHASH",	do_rehash,	&fRehash,	STAT_HELP_REHASH,	-2,	-2 },
	{ "RESTART",	do_restart,	&fRestart,	STAT_HELP_RESTART,	-2,	-2 },
        { "SHUTDOWN",	do_shutdown,	&fShutDown,	STAT_HELP_SHUTDOWN,	-2,	-2 },
	{ NULL }
};

void do_statserv(char *source, int c, char *buf)
{
    char *cmd = strtok(buf, " ");
    User *u = finduser(source);

    if (!cmd || !u) {
	SockWrite(":%s %s :ERROR in do_statserv()", ssNick, WALLOPS);
	log("Internal error: user = '%s' | command = '%s'", u ? u->nick : "<Unknown>", cmd ? cmd : "<Unknown>");
	return;
    }

    if (cmd[0]==1) {
	ctcp_reply(source, ssNick, cmd);
    } else if (!strcasecmp("HELP", cmd)) {
	ss_helpcount++;
	run_help(ssNick, u, cmds, STAT_HELP);
    } else {
	run_cmd(ssNick, u, cmds, buf);
    }
}

void do_ss_helpops(User *u, char *topic)
{
    strtok(topic, " ");
    run_help(ssNick, u, cmds, STAT_HELP);
}

static void stat_time(char *source, User *u)
{
    struct tm tm;
    time_t actu = time(NULL);
    char from[125], to[125];

    tm = *localtime(&actu);
    strftime(from, 125, getstring(u, FULL_TIME), &runsince_tm);
    strftime(to, 125, getstring(u, FULL_TIME), &tm);
    notice_lang(ssNick, u, STAT_FROM_TO, from, to);
}

static void do_map(User *u)
{
    char *view = malloc(sizeof(char *) * (servernum + 1));

    ss_mapcount++;
    memset(view, 0, sizeof(char *) * (servernum + 1));
    notice_lang(ssNick, u, STAT_MAP_LIST);
    show_server(ssNick, u, servmap, view);
    free(view);
}

static void do_servers(User *u)
{
    char *mask = strtok(NULL, " ");

    ss_serverscount++;
    showallservercount = 0;
    notice_lang(ssNick, u, STAT_SERVERS_LIST);
    if (show_all_servers(ssNick, u, servmap, mask)) {
	notice_lang(ssNick, u, END_OF_LIST);
    } else {
	notice_lang(ssNick, u, NO_MATCH);
    }
}

static void do_ss_server(User *u)
{
    char *name = strtok(NULL, " ");
    char cmax[125], lastsplit[125]; /*, longsplit[125]; */
    struct tm tm;

    ss_servercount++;
    if (!name) {
	syntax_error(ssNick, u, "SERVER", STAT_SERVER_SYNTAX);
    } else {
	Server *server = find_server_match(servmap, name);
	if (!server) {
	    notice_lang(ssNick, u, STAT_SERVER_UNKNOWN, name);
	} else if (server->connected==2 || IgnoreServer(server)) {
	    notice_lang(ssNick, u, STAT_SERVER_NO_STAT, server->name);
	} else {
            notice_lang(ssNick, u, STAT_SERVER, server->name);
	    stat_time(ssNick, u);

	    if (server->timemaxcount) {
		tm = *localtime(&server->timemaxcount);
		strftime(cmax, 125, getstring(u, FULL_TIME), &tm);
	    } else strcpy(cmax, "XX");
	    if (server->timelastsplit) {
		tm = *localtime(&server->timelastsplit);
		strftime(lastsplit, 125, getstring(u, FULL_TIME), &tm);
	    } else strcpy(lastsplit, "XX");
	    if (!server->connected)
		notice_lang(ssNick, u, STAT_SERVER_NO_RESPONSE);
	    notice_lang(ssNick, u, STAT_SERVER_CONTENT, server->desc, server->count, server->maxcount,
		cmax, server->average[0], server->average[1], server->average[2], server->average[3],
		server->totalcount, server->opercount, server->count ? server->opercount * 100 / server->count : 0,
		server->maxopercount, server->killcount, server->servkillcount, lastsplit);
	    notice_lang(ssNick, u, STAT_SERVER_END);
	}
    }
}

static void do_domain(User *u)
{
    char *name = strtok(NULL, " ");
    char to[125], from[125], cmax[125];
    struct tm tm;

    ss_domaincount++;
    if (!name) {
	syntax_error(ssNick, u, "DOMAIN", STAT_DOMAIN_SYNTAX);
    } else {
	Domain *domain = find_domain(name);
	if (!domain) {
	    notice_lang(ssNick, u, STAT_DOMAIN_UNKNOWN, name);
	} else {
	    notice_lang(ssNick, u, STAT_DOMAIN, name);
	    stat_time(ssNick, u);
	    tm = *localtime(&domain->timemaxcount);
	    strftime(cmax, 125, getstring(u, FULL_TIME), &tm);
	    tm = *localtime(&domain->firstlogin);
	    strftime(to, 125, getstring(u, FULL_TIME), &tm);
	    tm = *localtime(&domain->lastlogin);
	    strftime(from, 125, getstring(u, FULL_TIME), &tm);
	    notice_lang(ssNick, u, STAT_DOMAIN_CONTENT, domain->country ? domain->country : "Unknown", domain->count,
		domain->maxcount, cmax, domain->average[0], domain->average[1], domain->average[2], domain->average[3],
		domain->totalcount, to, from);
	    notice_lang(ssNick, u, STAT_DOMAIN_END);
	}
    }
}

static void do_netstats(User *u)
{
    char cmax[125], smax[125], chmax[125], dailycmax[125], dailychmax[125];
    struct tm tm;

    ss_netstatscount++;
    tm = *localtime(&maxusertime);
    strftime(cmax, 125, getstring(u, FULL_TIME), &tm);
    tm = *localtime(&timemaxservercount);
    strftime(smax, 125, getstring(u, FULL_TIME), &tm);
    tm = *localtime(&maxchanneltime);
    strftime(chmax, 125, getstring(u, FULL_TIME), &tm);
    tm = *localtime(&dailymaxusertime);
    strftime(dailycmax, 125, getstring(u, FULL_TIME), &tm);
    tm = *localtime(&dailymaxchanneltime);
    strftime(dailychmax, 125, getstring(u, FULL_TIME), &tm);

    notice_lang(ssNick, u, STAT_NET, AdminInfo);
    stat_time(ssNick, u);
    notice_lang(ssNick, u, STAT_NET_CONTENT, usercount + invusercount, maxusercount, cmax, netaverage[0], netaverage[1],
		netaverage[2], netaverage[3], resolvecount, servercount, maxservercount, smax, splitcount, channelcount,
		maxchannelcount, chmax, totalkillcount, totalservkillcount, dailytotalusercount, dailymaxusercount, dailycmax,
		dailymaxchannelcount, dailychmax, dailysplitcount, dailykillcount, dailyservkillcount);
    notice_lang(ssNick, u, STAT_NET_END);
}

static void do_operlist(User *u)
{
    ss_operlistcount++;

    notice_lang(ssNick, u, STAT_OPER);
    if (opercount) {
	int i;
	User *user;
	for (i=0; i < opercount; i++) {
	    user = finduser(operlist[i]);
	    if (!user) {
		log("do_operlist: Found an unknown oper '%s'", operlist[i]);
		return;
	    }
	    SockWrite(":%s NOTICE %s :%d  \2%s\2%s%s%s%s", ssNick, nickyxx(u), i+1,
		operlist[i], user->server ? " - ":"", user->server ? user->server->name : "",
		user->flags ? " - ":"", user->flags ? user->flags:"");
	}
	notice_lang(ssNick, u, STAT_OPER_END);
    } else {
	notice_lang(ssNick, u, STAT_OPER_NONE);
    }
}

static void do_set_language(User *u)
{
    char *lang = strtok(NULL, " ");
    int i, langnum;

    ss_languagecount++;
    if (lang)
	langnum=atoi(lang)-1;

    if (lang && strspn(lang, "0123456789") != strlen(lang)) {
	syntax_error(ssNick, u, "LANGUAGE", STAT_LANGUAGE_SYNTAX);
    } else if (!lang || (langnum=atoi(lang)-1) < 0 || langnum > LanguageNum) {
	notice_lang(ssNick, u, STAT_LANGUAGE_LIST);
	for (i=0; filelang[i]; i++)
	    SockWrite(":%s NOTICE %s :%d  %s", ssNick, nickyxx(u), i+1, FlyLang[i][0]);
	notice_lang(ssNick, u, STAT_LANGUAGE_END);
    } else {
	u->language = langnum;
	notice_lang(ssNick, u, STAT_LANGUAGE_SELECT, FlyLang[langnum][0]);
    }
}

static void do_identify(User *u)
{
    char *login = strtok(NULL, " ");
    char *pass = strtok(NULL, " ");

    ss_identifycount++;
    if (!pass) {
	syntax_error(ssNick, u, "IDENTIFY", STAT_IDENTIFY_SYNTAX);
    } else {
	if (u->flags)
	    User_UnIdentify(u);
	if (accessflags) {
	    /* User_Identify s'occupe de tout le reste */
	    User_Identify(u, login, pass);
	} else {
	    /* La commande IDENTIFY est bien entendu non utilisable si il n'y pas
	     * d'accs de definis dans le stat.conf
	     */
	    notice_lang(ssNick, u, STAT_IDENTIFY_DISABLED);
	}
    }
}

static void do_removeserver(User *u)
{
    char *name = strtok(NULL, " ");
    Server *server;

    ss_removeservercount++;
    if (!name) {
	syntax_error(ssNick, u, "REMOVESERVER", STAT_REMOVESERVER_SYNTAX);
    } else if (!(server=find_server(servmap, name))) {
	notice_lang(ssNick, u, STAT_SERVER_UNKNOWN, name);
    } else if (server->connected) {
	notice_lang(ssNick, u, STAT_REMOVESERVER_ERROR);
    } else {
	log("'%s' used REMOVESERVER command on '%s'", name);
	if (server->leaf)
	    remove_server_leaf(server);
	remove_one_server(server);

	notice_lang(ssNick, u, STAT_REMOVESERVER_DONE, name);
	if (WallOnOperCommand)
	    SockWrite(":%s %s :\2%s\2 used REMOVESERVER command on %s", ssNick, WALLOPS, u->nick, name);
	if (!servernum) {
	    SockWrite(":%s %s :Bogus servernum in m_squit()!", ServerName, WALLOPS);
	    log("Bogus servernum (%d)", servernum);
	    servernum = 1;
	}

    }
}

static void do_rehash(User *u)
{
    ss_rehashcount++;
    quitflystats = 12;
    if (WallOnOperCommand)
	SockWrite(":%s %s :\2%s\2 used REHASH command", ssNick, WALLOPS, u->nick);
    log("'%s!%s@%s' used REHASH command", u->nick, u->username, u->host);
    SockWrite(":%s NOTICE %s :Rehashing Stats conf. file...", ssNick,  nickyxx(u));
}

static void do_restart(User *u)
{
    quitflystats = 13;
    if (WallOnOperCommand)
	SockWrite(":%s %s :\2%s\2 used RESTART command", ssNick, WALLOPS, u->nick);
    log("%s!%s@%s' used RESTART command", u->nick, u->username, u->host);
    SockWrite(":%s NOTICE %s :Restarting StatServ...", ssNick, nickyxx(u));
}

static void do_shutdown(User *u)
{
    quitflystats = 14;
    if (WallOnOperCommand)
	SockWrite(":%s %s :\2%s\2 used SHUTDOWN command", ssNick, WALLOPS, u->nick);
    log("'%s!%s@%s' used SHUTDOWN command", u->nick, u->username, u->host);
    SockWrite(":%s NOTICE %s :StatServ Terminating...", ssNick, nickyxx(u));
}
