/* FlyStats! is copyright (c) 1999-2000 CRESTO Sylvain <tost>.
 * E-mail: <tost@softhome.net>
 * Contact: #flystats on irc.kewl.org
 *
 * This program is free but copyrighted software; see the file COPYING for
 * details.
 */

#include "extern.h"

#define HASH(nick)      (((nick)[0]&31)<<5 | ((nick)[1]&31))
User *userlist[1024];
char **operlist = NULL;
long usercount = 1, invusercount = 0, maxusercount = 0, totalkillcount = 0, dailymaxusercount = 0, dailytotalusercount = 0,
	channelcount = 0, maxchannelcount = 0, dailymaxchannelcount = 0;
int opercount = 0, maxopercount = 0, dailykillcount = 0;
time_t maxusertime, maxopertime, dailymaxusertime, maxchanneltime, dailymaxchanneltime;

User *identified = NULL;

User *finduser(const char *nick)
{
    User *u = userlist[HASH(nick)];

    while (u && strcasecmp(u->nick, nick))
	u = u->next;

    return u;
}

#ifdef IRC_UNDERNET
User *finduser_yxx(const char *nick)
{
    Server *server = find_server_yxx(servmap, nick[0]);
    User *u;

    if (!server) {
	log("Unable to find server (P10 %s)", nick);
	return NULL;
    }
    u = server->userlist[*(nick+1)-48];
    while (u && strcmp(nick, u->yxx))
	u = u->yxx_next;
    return u;
}
#endif

void remove_ircop(User *u)
{
    if (u->server && !IgnoreServer(u->server)) {
	int i;
	char **oper;
	for (oper = operlist, i = 0; i < opercount; oper++, i++)
	if (!strcmp(*oper, u->nick))
	    break;
	free(*oper);
	opercount--;
	if (i < opercount)
	    memmove(oper, oper+1, (opercount-i) * sizeof(char *));
	if (opercount) {
	    operlist = realloc(operlist, opercount * sizeof(char *));
	} else {
	    free(operlist);
	    operlist = NULL;
	}
    }
    u->server->opercount--;
    u->mode &= ~UMODE_O;
}

/* traite une commande NICK d'un nouveau utilisateur
 * 0 - nick
 * 1 - username
 * 2 - hostname
 * 3 - servername (yxx avec l'ircu 2.10.*)
 * 4 - signon
 */
User *do_newnick(char **data)
{
    User *u = calloc(sizeof(User), 1), **list
#ifdef IRC_UNDERNET
	, **yxx_list
#endif
	;
    int resolve;
    Server *server;

#ifdef ALLOW_DEBUG
    if (debug)
	log("New user: %s!%s@%s", data[0], data[1], data[2]);
#endif
    u->nick = strdup(data[0]);
    u->username = strdup(data[1]);
    u->host = strdup(data[2]);
    if (!(u->signon = atol(data[4]))) {
	log("New user '%s!%s@%s' (%) as NULL signon, changed it to current time", data[0], data[1], data[2], data[4]);
	u->signon = time(NULL);
    }
    resolve = (strspn(data[2], "0123456789.") != strlen(data[2]));

	/* detecte le language */
    u->language = detectlang(data[2]);
    if (u->language==-1)
	u->language=DefaultLang;

	/* mets a jours les stats sur le nombre d'utilisateur */
    dailytotalusercount++;
    if (++usercount+invusercount > dailymaxusercount) {
	dailymaxusercount = usercount+invusercount;
	dailymaxusertime = time(NULL);
	if (dailymaxusercount > maxusercount) {
	    maxusercount = dailymaxusercount;
	    maxusertime = dailymaxusertime;
	}
	if (LogMaxUser)
	    log("user: New maximum user count: %d", maxusercount);
    }

	/* mets a jour les stats sur le domaine */
    if (resolve) {
	char *s = u->host+strlen(u->host)-1;
	int i = 0;
	Domain *domain;
	while (s!=u->host-1) {
	    if (*--s=='.') {
		i++;
		if (i==2) {
		    s++;
		    break;
		}
	    }
	}
	if (!(domain = find_domain(s)))
	    domain = new_domain(s);
	domain_adduser(domain);
	u->domain = domain;
	resolvecount++;
    }
	/* mets  jour les stats sur le serveur */
#ifdef IRC_UNDERNET
    if ((server = find_server_yxx(servmap, *data[3]))) {
#else
    if ((server = find_server(servmap, data[3]))) {
#endif
	server->count++;
	server->totalcount++;
	if (server->count>server->maxcount) {
	    server->maxcount = server->count;
	    server->timemaxcount = time(NULL);
	}
	u->server = server;
    } else {
	/* a ne devrait jamais arriver */
#ifdef IRC_UNDERNET
	log("User %s!%s@%s is on an unknown server, memory leaks", u->nick, u->username, u->host);
	return NULL;
#else
	log("User %s!%s@%s is on unknown server '%s'", u->nick, u->username, u->host, data[5]);
#endif
    }

#ifdef IRC_UNDERNET
    yxx_list = &server->userlist[*(data[3]+1)-48];
    u->yxx_next = *yxx_list;
    if (*yxx_list)
        (*yxx_list)->yxx_prev = u;
    *yxx_list = u;
    strcpy(u->yxx, data[3]);
#endif
    list = &userlist[HASH(data[0])];
    u->next = *list;
    if (*list)
        (*list)->prev = u;
    *list = u;

    return u;
}

/* Un user change de nick */
void do_changenick(char *source, char *nick)
{
    User *user =
#ifdef IRC_UNDERNET
	finduser_yxx(source);
#else
	finduser(source);
#endif
    if (!user) {
	log("Received change NICK for unknown user '%s' to '%s'", source, nick);
    } else {
	User **list;
#ifdef ALLOW_DEBUG
	if (debug)
	    log("User '%s' are now known as '%s'", user->nick, nick);
#endif
	if (user->prev) {
	    user->prev->next = user->next;
	} else {
	    userlist[HASH(user->nick)] = user->next;
	}
	if (user->next) {
	    user->next->prev = user->prev;
	}
	free(user->nick);
	user->nick = strdup(nick);

	list = &userlist[HASH(user->nick)];
	user->next = *list;
	user->prev = NULL;
	if (*list)
	    (*list)->prev = user;
	*list = user;
	    /* mets  jour l liste des opers */
	if (user->mode & UMODE_O) {
	    int i;
	    for (i=0; i < opercount; i++)
	    if (!strcmp(operlist[i], source)) {
		free(operlist[i]);
		operlist[i] = strdup(nick);
		break;
	    }
	}
    }
}

/* Un user quit (quit ou kill) */
void do_quit(char *nick, char *quitmessage)
{
    User *user = finduser(nick);
    if (!user) {
	/* Ne pas envoyer de log si c'est un pesudo client qui QUIT (lors d'un rehash par exemple) */
	if (!pseudo_client(nick))
	    log("Received QUIT for unknown user '%s'", nick);
    } else {
#ifdef ALLOW_DEBUG
	if (debug)
	    log("User '%s' has quit (%s)", user->nick, quitmessage);
#endif
#ifdef DETECT_LOCALKILL
	if (!strncmp(quitmessage, LOCALKILL, strlen(LOCALKILL))) {
	    /* c'est vraiment laid mais c'est la seul faon d'intercepter un local kill */
	    if (user->server)
		user->server->killcount++;
	    totalkillcount++;
	    dailykillcount++;
	}
#endif
	if (user->domain) {
	    user->domain->count--;
	    resolvecount--;
	}
	if (user->server) {
	    user->server->count--;
#ifdef IRC_UNDERNET
	    if (user->yxx_prev)
		user->yxx_prev->yxx_next = user->yxx_next;
	    else {
		user->server->userlist[*(user->yxx+1)-48] = user->yxx_next;
	    }
	    if (user->yxx_next)
		user->yxx_next->yxx_prev = user->yxx_prev;
#endif
	}
#ifdef USETHREAD
	/* on vire l'host de la liste de verification des proxy */
	if (user->proxy_prev || firstproxycheckuser==user) {
	    if (user->proxy_next) {
		user->proxy_next->proxy_prev = user->proxy_prev;
	    } else lastproxycheckuser = user->proxy_prev;
	    if (user->proxy_prev) {
		user->proxy_prev->proxy_next = user->proxy_next;
	    } else firstproxycheckuser = user->proxy_next;
	}
#endif
	/* on vire l'user de la liste des identifis si il y est */
	if (user->flags) {
	    if (user->ident_next)
		user->ident_next->ident_prev = user->ident_prev;
	    if (user->ident_prev) {
		user->ident_prev->ident_next = user->ident_next;
	    } else identified = user->ident_next;
	}
	if (user->mode & UMODE_I) {
	    invusercount--;
	} else {
	    usercount--;
	}
	if (user->mode & UMODE_O)
	    remove_ircop(user);

	if (user->prev)
	    user->prev->next = user->next;
	else {
	    userlist[HASH(user->nick)] = user->next;
	}
	if (user->next)
	    user->next->prev = user->prev;
        free(user->nick);
        free(user->username);
        free(user->host);
	free(user);
    }
}

void do_umode(char *nick, char *mode)
{
    User *u = finduser(nick);
    char *s = mode;
    int add = 1;

    if (!u) {
	log("Received MODE '%s' for unknown user '%s'", mode, nick);
	return;
    }

    while (*s && *s!=32) {
	switch (*s++) {
	    case '+':	add = 1; break;
	    case '-':	add = 0; break;
	    case 'i':	/* mode invisible */
			if (add) {
			    u->mode |= UMODE_I;
			    usercount--;
			    invusercount++;
			} else {
			    u->mode &= ~UMODE_I;
			    usercount++;
			    invusercount--;
			}
			break;
	    case 'o':   /* mode opers */
			if (add) {
			    if (u->server && !IgnoreServer(u->server)) {
				if (WallOnNewOper)
				    SockWrite(":%s %s :\2%s\2 is now an IRC operator.", ssNick, WALLOPS, nick);
				u->mode |= UMODE_O;
				opercount++;
				if (opercount > maxopercount) {
				    maxopercount = opercount;
				    maxopertime = time(NULL);
				}
				u->server->opercount++;
				if (u->server->opercount > u->server->maxopercount)
				    u->server->maxopercount=u->server->opercount;
				operlist = realloc(operlist, sizeof(char *) * opercount);
				operlist[opercount-1] = strdup(u->nick);
			    }
			    if (NotifyOpersMissingServer && actualsplit)
				notice_lang(ssNick, u, NOTIFY_SERVER_SPLIT, actualsplit);
			} else {
			    remove_ircop(u);
			}
			break;
	}
    }
}

/* Renvois 1 si c'est un ircop 0 sinon */
int is_oper(User *u)
{
    return (u->mode & UMODE_O);
}

/* Renvois 1 si l'user  les bon flags, 0 sinon */
int is_access(User *u, int flagrequired)
{
    if (u->flags) {
	char *s = u->flags;
	/* On regarde si le flag requis est present dans ceux de l'utilisateur */
	while (*s) {
	    if (*(s++)==flagrequired)
		return 1;
	}
    }
    return 0;
}

/* Desidentifis un utilisateur */
void User_UnIdentify(User *u)
{
#ifdef ALLOW_DEBUG
    if (debug)
	log("UnIdentify user '%s!%s@%s'", u->nick, u->username, u->host);
#endif
    if (u->ident_prev) {
	u->ident_prev->next = NULL;
    } else identified = u->ident_next;
    if (u->ident_next) {
	u->ident_next->ident_prev = u->ident_prev;
    }
    u->ident_prev = u->ident_next = NULL;
    if (u->flags) {
	free(u->flags);
	u->flags = NULL;
    } else log("User '%s' already have NULL levels (?)", u->nick);
}

/* Identifis un utilisateur
 * Renvois 1 si c'est oki, 0 sinon
 */
int User_Identify(User *u, char *login, char *pass)
{
    AccessFlags *af;
    char buf[250];
    int hostmis = 0;

    snprintf(buf, 250, "%s!%s@%s", u->nick, u->username, u->host);
    for (af = accessflags; af; af=af->next) {
	if (!strcmp(af->login, login) && !strcmp(af->password, pass)) {
	    if (truematch_wild(af->mask, buf)) {
		log("'%s' identified with flags '%s'", buf, af->flags);
		u->flags = strdup(af->flags);
		if (WallOnIdentify)
		    SockWrite(":%s %s :\2%s\2 identified for '%s/%s'", ssNick, WALLOPS, u->nick, af->login, af->flags);
		notice_lang(ssNick, u, STAT_IDENTIFY_DONE, af->login, af->flags);
		u->ident_next=identified;
		if (identified)
		    identified->ident_prev = u;
		identified = u;
		return 1;
	    } else hostmis = 1;
	}
    }
    log("'%s' failed IDENTIFY, %s mismatch", buf, hostmis ? "Host" : "Login or Password");
    if (WallOnFailedIdentify)
	SockWrite(":%s %s :\2%s\2 failed to identify, %s mismatch", ssNick, WALLOPS, u->nick,
		hostmis ? "Host" : "Login or Password");
    notice_lang(ssNick, u, hostmis ? STAT_IDENTIFY_FAILED_HOST : STAT_IDENTIFY_FAILED_LOGIN_PASSWORD);
    return 0;
}

#ifndef IRC_UNDERNET
/* Pour l'ircu 2.10.x cette fonction est presente dans server.c
 */
void remove_server_users(Server *server)
{
    int i = 0;
    User *current = NULL, *u;

	/* Vivement la gestion du NOQUIT :)
	 */
    do {
	while (i < 1024 && !current)
	    current = userlist[i++];
	while (current) {
	    u = current->next;
	    if (current->server==server)
		do_quit(current->nick, "NOQUIT");
	    current = u;
	}
    } while (i!=1024);
}
#endif
